/**
  * NetworkInterface
  * Auteur : G.Burri
  * version 1.0
  */
  
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

import java.awt.Dimension;
 

/**
  * class NetException
  * Peut-tre dclanch lors d'erreur de communication.
  */ 
class NetException extends Exception
{   
   public NetException(String msg)
   {
      super(msg);
   }
}

/**
  * class Player
  * Permet de communiquer avec un serveur JTron.
  * Version du protocole : 1
  */
class NetworkInterface
{
   //la version du protocole, est incrment lors de modification
   final private byte protocol = (byte)0x01;
        
    private InputStream in;
    private OutputStream out;
    
    private InetAddress address;
    
    private int port;
    
    //buffer pour la reception des donnes
    private byte[] buffer = new byte[255];   

   /**
     * Constructeur.
     * InetAddress : l'adresse du serveur
     * port : le port du serveur
     */
    public NetworkInterface(InetAddress i, int port)
    {
        this.address = i;
        this.port = port;
    }
    
   /**
     * Etablit la connexion avec le serveur.
     */
    public void connect() throws NetException
    {
        try{
            Socket socket = new Socket(this.address, this.port);
            socket.setTcpNoDelay(true);
            this.in = socket.getInputStream();
            this.out = socket.getOutputStream();
        }catch(IOException e)
        {
            throw new NetException("Cannot connect to the server");
        };
    }    
    
   /**
     * Attends la demande d'infos (attends message numro 01).
     */
    public void waitInfos() throws NetException
    {
        try{
            this.in.read(this.buffer, 0, 1);
        }catch(IOException e){
            throw new NetException("Connexion problem...");
        };
              
        if(this.buffer[0] != 0x01)
            throw new NetException("Protocol problem message numero 1");
    }
    

   /**
     * Envois les informations (message numro 02).
     * name : le nick
     */
    public Dimension sendInfos(String name) throws NetException
    {
        byte[] n = name.getBytes();
        
        byte[] buff = new byte[n.length + 3];
        
        buff[0] = 0x02;
        buff[1] = this.protocol;
        
        for(int i=0; i<n.length; i++)
            buff[i+2] = n[i];
        
        buff[n.length+2] = 0x7F;
        
        try{
            this.out.write(buff);
            this.in.read(this.buffer, 0, 3);
        }catch(IOException e){
            throw new NetException("Connexion problem...");
        };
        
        if(this.buffer[0] != 0x03)
            throw new NetException("Protocol problem message numero 3");
        
        return new Dimension(buffer[1], buffer[2]);
    }
    
   /**
     * Attends un nouveau round,
     * comprends les messages "ping-pong" et 05.
     * retourne : les joueurs qui vont participer  ce round
     * sous la forme d'un vecteur de Player (voir classe Player)
     */
    public Vector waitNewRound() throws NetException
    {
        try{
            for(;;)
            {
                this.buffer = new byte[255];
                this.in.read(this.buffer);
                
                if(this.buffer[0] == 0x0B)
                {
                    byte[] buff = { 0x0C };
                    this.out.write(buff);
                }
                else break;
            }
        }catch(IOException e){
            throw new NetException("Connexion problem...");
        };
        
        if(this.buffer[0] != 0x05)
            throw new NetException("Protocol problem...");
                        
        Vector players = new Vector((int)this.buffer[1]);
         
        int curseur = 2;
        for(int i=0; i<this.buffer[1]; i++)
        {
            Position pos = new Position((int)this.buffer[curseur], (int)this.buffer[curseur+1]);
            curseur += 2;
            int id = (int)this.buffer[curseur++];
            
            byte[] pseudo = new byte[20];
            
            //lit le pseudo
            int j=0;
            while(this.buffer[curseur] != 0x7F)
            {
                pseudo[j] = this.buffer[curseur];
                j++;
                curseur++;
            }
            curseur++;
            
            String name = new String(pseudo);
            name = name.trim();

            players.add(new Player(pos, id, name));            
        }

        return players;        
    }
    
   /**
     * Attends que le round commence (que tous les joueurs soit prt),
     * comprends les messages 06 et 07.
     */
    public void waitRoundStart() throws NetException
    {
        byte[] buff = { 0x06 };
        try{
            this.out.write(buff);
            this.in.read(this.buffer, 0, 1);
        }catch(IOException e){
            throw new NetException("Connexion problem...");
        };
        
        if(this.buffer[0] != 0x07)
            throw new NetException("Protocol problem...");        
    }
    
   /**
     * Attends une demande de mouvement de la part du serveur
     * (message 08).
     */
    public void waitMoveAsk() throws NetException
    {
        try{
            this.in.read(this.buffer, 0, 1);
        }catch(IOException e){
            throw new NetException("Connexion problem...");
        };
        
        if(this.buffer[0] != 0x08)
            throw new NetException("Protocol problem...");     
    }
    
   /**
     * Envois une direction.
     * direction : la direction, peut prendre les valeurs suivantes :
     *  0x01 : nord 
     *  0x02 : sude
     *  0x03 : est
     *  0x04 : ouest
     *  0x00 : aucun changement de direction
     */
    public void sendMove(byte direction) throws NetException
    {
        byte[] buff = new byte[2];
        buff[0] = 0x09;
        buff[1] = direction;
        
        try{
            this.out.write(buff);
        }catch(IOException e){
            throw new NetException("Connexion problem...");
        };
    }
    
   /**
     * Attends un rafraichissement de l'tat du jeu.
     * Retourne : l'tat du jeu (voir classe State)
     */
    public State waitRefresh() throws NetException
    {      
        try{
            this.in.read(this.buffer);
        }catch(IOException e){
            throw new NetException("Connexion problem...");
        };
            
        if(this.buffer[0] != 0x0A)
            throw new NetException("Protocol problem...");
       
        State state = new State(buffer[1]);        
        
        for(int i=2; this.buffer[i]!=0x7F; i+=3)
        {
            state.addPlayer(new PlayerBase(new Position(this.buffer[i], this.buffer[i+1]), this.buffer[i+2]));
        }
        
        return state;
    }
    
    /**
      * Affiche le buffer courant (DEBUG)
      */
    private void printBuffer()
    {
        for(int i=0; i<20; i++)
            System.out.print(this.buffer[i] + ", ");    
        System.out.println("");
    }   
}

